// bncView.cpp : implementation of the CBncView class
//

#include "stdafx.h"
#include "NewTmpl.h"

#include "bncDoc.h"
#include "bncView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ABS(x) ((x) < 0? -(x) : (x) > 0? (x) : 0)


/////////////////////////////////////////////////////////////////////////////
// CBncView

IMPLEMENT_DYNCREATE(CBncView, CView)

BEGIN_MESSAGE_MAP(CBncView, CView)
	//{{AFX_MSG_MAP(CBncView)
	ON_COMMAND(IDM_BLACK, OnColor)
	ON_UPDATE_COMMAND_UI(IDM_BLACK, OnUpdateColor)
	ON_COMMAND(IDM_CUSTOM, OnCustomColor)
	ON_COMMAND(IDM_SLOW, OnSlow)
	ON_UPDATE_COMMAND_UI(IDM_SLOW, OnUpdateSlow)
	ON_COMMAND(IDM_FAST, OnFast)
	ON_UPDATE_COMMAND_UI(IDM_FAST, OnUpdateFast)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(IDM_BLUE, OnColor)
	ON_UPDATE_COMMAND_UI(IDM_BLUE, OnUpdateColor)
	ON_COMMAND(IDM_GREEN, OnColor)
	ON_UPDATE_COMMAND_UI(IDM_GREEN, OnUpdateColor)
	ON_COMMAND(IDM_RED, OnColor)
	ON_UPDATE_COMMAND_UI(IDM_RED, OnUpdateColor)
	ON_COMMAND(IDM_WHITE, OnColor)
	ON_UPDATE_COMMAND_UI(IDM_WHITE, OnUpdateColor)
	ON_UPDATE_COMMAND_UI(IDM_CUSTOM, OnUpdateColor)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBncView construction/destruction

CBncView::CBncView()
{
	// TODO: add construction code here

}

CBncView::~CBncView()
{
}

BOOL CBncView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBncView drawing

void CBncView::OnDraw(CDC* pDC)
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

/////////////////////////////////////////////////////////////////////////////
// CBncView diagnostics

#ifdef _DEBUG
void CBncView::AssertValid() const
{
	CView::AssertValid();
}

void CBncView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBncDoc* CBncView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBncDoc)));
	return (CBncDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBncView message handlers

void CBncView::OnColor() 
{
	CBncDoc* pDoc= GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->m_nIDColor = LOWORD(GetCurrentMessage()->wParam);
	pDoc->SetBallColor();
	MakeNewBall();
}

void CBncView::OnUpdateColor(CCmdUI* pCmdUI) 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->SetCheck(pCmdUI->m_nID == pDoc->m_nIDColor);
}

void CBncView::OnCustomColor() 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog dlgColor(pDoc->m_clrBall);
	if (dlgColor.DoModal() == IDOK)
	{
		pDoc->m_nIDColor = IDM_CUSTOM;
		pDoc->SetCustomBallColor(dlgColor.GetColor());
	}
	MakeNewBall();
}

void CBncView::OnSlow() 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->m_bFastSpeed= FALSE;
	ChangeSpeed();
}

void CBncView::OnUpdateSlow(CCmdUI* pCmdUI) 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pCmdUI->SetCheck(!pDoc->m_bFastSpeed);
}

void CBncView::OnFast() 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->m_bFastSpeed= TRUE;
	ChangeSpeed();
}

void CBncView::OnUpdateFast(CCmdUI* pCmdUI) 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->SetCheck(pDoc->m_bFastSpeed);
}

void CBncView::ChangeSpeed()
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
// re-create the timer
	KillTimer(1);
	if (!SetTimer(1, pDoc->m_bFastSpeed ? 0 : 100, NULL))
	{
		MessageBox(_T("Not enough timers available for this window."),
				_T("MDI:Bounce"), MB_ICONEXCLAMATION | MB_OK);
		DestroyWindow();
	}
}

int CBncView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!SetTimer(1, 100 /*start slow*/, NULL))
	{
		MessageBox(_T("Not enough timers available for this window."),
				_T("MDI:Bounce"), MB_ICONEXCLAMATION | MB_OK);

		// signal creation failure...
		return -1;
	}

	CDC* pDC = GetDC();
	CPoint aspect;
	aspect.x = pDC->GetDeviceCaps(ASPECTX);
	aspect.y = pDC->GetDeviceCaps(ASPECTY);
	ReleaseDC(pDC);
	pDoc->m_ptPixel= aspect;

	// Note that we could call MakeNewBall here (which should be called
	// whenever the ball's speed, color or size has been changed), but the
	// OnSize member function is always called after the OnCreate. Since
	// the OnSize member has to call MakeNewBall anyway, we don't here.

	return 0;
}

void CBncView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	LONG lScale;
	CPoint center, ptPixel;

	center.x = cx >> 1;
	center.y = cy >> 1;
	center.x += cx >> 3; // make the ball a little off-center
	pDoc->m_ptCenter= center;
	
	CSize radius, move;

	ptPixel= pDoc->m_ptPixel;

	lScale = min((LONG)cx * ptPixel.x,
		(LONG)cy * ptPixel.y) >> 4;

	radius.cx = (int)(lScale / ptPixel.x);
	radius.cy = (int)(lScale / ptPixel.y);
	pDoc->SetBallRadius(radius);

	move.cx = max(1, radius.cy >> 2);
	move.cy = max(1, radius.cy >> 2);
	pDoc->m_sizeMove= move;

	MakeNewBall();
}

void CBncView::OnTimer(UINT nIDEvent) 
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CBitmap* pbmBall;
	pbmBall= &pDoc->m_bmBall;

	if (pbmBall->m_hObject == NULL)
		return;     // no bitmap for the ball

	CRect rcClient;
	GetClientRect(rcClient);

	CClientDC dc(this);
	CBitmap* pbmOld = NULL;

	CDC dcMem;
	dcMem.CreateCompatibleDC(&dc);
	pbmOld = dcMem.SelectObject(pbmBall);

	CPoint center;
	CSize total, move, radius;

	center= pDoc->m_ptCenter;
	radius= pDoc->m_sizeRadius;
	total= pDoc->m_sizeTotal;
	move= pDoc->m_sizeMove;

	dc.BitBlt(center.x - total.cx / 2, center.y - total.cy / 2,
			total.cx, total.cy, &dcMem, 0, 0, SRCCOPY);

	
	center += move;
	pDoc->m_ptCenter= center;

	if ((center.x + radius.cx >= rcClient.right) ||
		(center.x - radius.cx <= 0))
	{
		move.cx = -move.cx;
	}

	if ((center.y + radius.cy >= rcClient.bottom) ||
		(center.y - radius.cy <= 0))
	{
		move.cy = -move.cy;
	}
	pDoc->m_sizeMove= move;

	dcMem.SelectObject(pbmOld);
	dcMem.DeleteDC();
}

void CBncView::MakeNewBall()
{
	CBncDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CSize radius, move, total;
	CBitmap* pbmBall;

	pbmBall= &pDoc->m_bmBall;
	radius= pDoc->m_sizeRadius;
	move= pDoc->m_sizeMove;
	
	total.cx = (radius.cx + ABS(move.cx)) << 1;
	total.cy = (radius.cy + ABS(move.cy)) << 1;
	pDoc->m_sizeTotal= total;

	if (pbmBall->m_hObject != NULL)
		pbmBall->DeleteObject();        // get rid of old bitmap

	CClientDC dc(this);
	CDC dcMem;
	dcMem.CreateCompatibleDC(&dc);

	pbmBall->CreateCompatibleBitmap(&dc, total.cx, total.cy);
	ASSERT(pbmBall->m_hObject != NULL);

	CBitmap* pOldBitmap = dcMem.SelectObject(pbmBall);

	// draw a rectangle in the background (window) color
	CRect rect(0, 0, total.cx, total.cy);
	CBrush brBackground(::GetSysColor(COLOR_WINDOW));
	dcMem.FillRect(rect, &brBackground);

	CBrush brCross(HS_DIAGCROSS, 0L);
	CBrush* pOldBrush = dcMem.SelectObject(&brCross);

	dcMem.SetBkColor(pDoc->m_clrBall);
	dcMem.Ellipse(ABS(move.cx), ABS(move.cy),
		total.cx - ABS(move.cx),
		total.cy - ABS(move.cy));

	dcMem.SelectObject(pOldBrush);
	dcMem.SelectObject(pOldBitmap);
	dcMem.DeleteDC();
}
